/*
 * Copyright (C) 2014 Yanjun Jiang <jiangy76@mcmaster.ca>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in
 *       the documentation and/or other materials provided with the distribution
 *     * Neither the name of the McMaster Centre for Software Certification nor the names
 *       of its contributors may be used to endorse or promote products derived
 *       from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
package ca.mcscert.jtet.eventbgenerator;

import ca.mcscert.jtet.expression.EventBGenerator;
import ca.mcscert.jtet.expression.VariableCollection;
import ca.mcscert.jtet.tablularexpression.*;

import java.util.List;

/**
 *
 * @author Yanjun Jiang <jiangy76@mcmaster.ca>
 */
final public class EventBTableGenerator {

    public EventBTableGenerator(VariableCollection variableDefinitions, String tableName, HierarchicalGrid leftGrid,  List<String> outputVariables, TwoDimensionalGrid outputGrid) {
        m_variableDefinitions = variableDefinitions;
        m_tableName = tableName;
        m_leftGrid  = leftGrid;
        m_outputVariables =  outputVariables;
        m_outputGrid = outputGrid;
    }

    public String generateEventBXml(){
        final String versionEncodingXml = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n";
        final String machineFileCfgPreXml = "<org.eventb.core.machineFile org.eventb.core.configuration=\"org.eventb.core.fwd;de.prob.units.mchBase\" version=\"5\">\n";
        final String machineFileCfgPostXml = "</org.eventb.core.machineFile>";

        //Step 1: Generate the Pre Xml
        m_output += versionEncodingXml;
        m_output += machineFileCfgPreXml;

        //Step 2: Generate the Variables & Invariants Xml
        m_output += m_eventBGenerator.GenerateVariablesDeclaration(m_variableDefinitions);

        //Step 3: Generate the Events Xml
        m_hierarchcialGridCheckerGenerator = new HierarchicalGridEventBGenerator(m_variableDefinitions, m_tableName, m_outputVariables, m_outputGrid);
        m_output  += HierarchcialGridEventBCheckerWalkerGenerator.GenerateCheckerFromGrid(m_leftGrid, m_hierarchcialGridCheckerGenerator);

        //Step 3: Generate the Post Xml
        m_output += machineFileCfgPostXml;

        return m_output;
    }

    String m_output = "";
    String m_tableName = "";
    HierarchicalGrid m_leftGrid;
    List<String> m_outputVariables;
    TwoDimensionalGrid m_outputGrid;
    VariableCollection m_variableDefinitions;
    EventBGenerator m_eventBGenerator = new EventBGenerator();
    HierarchcialGridCheckerGenerator m_hierarchcialGridCheckerGenerator;
}