/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package ca.mcscert.jtet.parsers;

import ca.mcscert.jtet.expression.BooleanVariableType;
import ca.mcscert.jtet.expression.EnumerationVariableType;
import ca.mcscert.jtet.expression.FixedPointVariableType;
import ca.mcscert.jtet.expression.RealVariableType;
import ca.mcscert.jtet.expression.Variable;
import ca.mcscert.jtet.expression.VariableCollection;
import ca.mcscert.jtet.parsers.VariableParserParser.Bool_typeContext;
import ca.mcscert.jtet.parsers.VariableParserParser.DefaulttypeContext;
import ca.mcscert.jtet.parsers.VariableParserParser.Fixed_typeContext;
import ca.mcscert.jtet.parsers.VariableParserParser.Real_typeContext;
import ca.mcscert.jtet.parsers.VariableParserParser.VarContext;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

import ca.mcscert.jtet.expression.VariableType;
import org.antlr.v4.runtime.ANTLRInputStream;
import org.antlr.v4.runtime.CommonTokenStream;
import org.antlr.v4.runtime.misc.NotNull;
import org.antlr.v4.runtime.tree.ParseTree;
import org.antlr.v4.runtime.tree.ParseTreeWalker;
import org.antlr.v4.runtime.tree.TerminalNode;

/**
 *
 * @author Matthew Dawson
 */
final public class VariableParser {

    public VariableCollection parseVariables(String variables) {
        // create a CharStream that reads from standard input
        ANTLRInputStream input = new ANTLRInputStream(variables) {

            @Override
            public int LA(int i) {
                return Character.toLowerCase(super.LA(i));
            }

        };

        // create a lexer that feeds off of input CharStream
        VariableParserLexer lexer = new VariableParserLexer(input);

        // create a buffer of tokens pulled from the lexer
        CommonTokenStream tokens = new CommonTokenStream(lexer);

        // create a parser that feeds off the tokens buffer
        VariableParserParser parser = new VariableParserParser(tokens);

        ParseTree tree = parser.varlist();
        ParseTreeWalker walker = new ParseTreeWalker();
        VariablesParserToVariables listener = new VariablesParserToVariables();
        walker.walk(listener, tree);

        return listener.m_variableCollection;
    }

    public void addCustomType(String typeName, VariableType type) {
        m_customTypes.put(typeName, type);
    }

    private final Map<String, VariableType> m_customTypes = new HashMap<String, VariableType>();

    private class VariablesParserToVariables extends VariableParserParserBaseListener {
        private String currentName;
        private VariableType currentType;
        private limitingExpressionData currentLimitingExpression;

        @Override
        public void enterVar(VarContext ctx) {
            currentName = ctx.ID().getText();
        }

        @Override
        public void enterDefaulttype(DefaulttypeContext ctx) {
            currentType = new RealVariableType();
        }

        @Override
        public void enterReal_type(Real_typeContext ctx) {
            currentType = new RealVariableType();
        }

        @Override
        public void enterBool_type(Bool_typeContext ctx) {
            currentType = new BooleanVariableType();
        }

        @Override
        public void enterUnknown_type(@NotNull VariableParserParser.Unknown_typeContext ctx) {
            currentType = m_customTypes.get(ctx.UNKNOWN_TYPE().getText());
            if (currentType == null) { // Assume this unknown.  For now, pretend a real is wanted.
                currentType = new RealVariableType();
            } else if (currentType instanceof EnumerationVariableType) {
                m_usedEnumerationTypes.add((EnumerationVariableType)currentType);
            }
        }

        @Override
        public void enterSubtype_var(@NotNull VariableParserParser.Subtype_varContext ctx) {
            currentLimitingExpression = new limitingExpressionData();
            currentLimitingExpression.variableName = ctx.ID().getText();
        }

        @Override
        public void enterPvs_expr(@NotNull VariableParserParser.Pvs_exprContext ctx) {
            currentLimitingExpression.expression = ctx.PVS_EXPR().getText();
        }

        @Override
        public void enterFixed_type(Fixed_typeContext ctx) {
            boolean signed = (ctx.NAT(0).getText().equals("0"))?false:true;
            int digits = Integer.parseInt(ctx.NAT(1).getText());
            int fractionPart = 0;
            TerminalNode fractionNode;
            if ((fractionNode = ctx.NAT(2)) != null) {
                fractionPart = Integer.parseInt(fractionNode.getText());
            }
            currentType = new FixedPointVariableType(signed, digits, fractionPart);
        }

        @Override
        public void exitVar(VarContext ctx) {
            m_variables.put(currentName, new Variable(currentName, currentType));
            if (currentLimitingExpression != null) {
                m_variableLimitingExpressions.put(currentName, currentLimitingExpression);
            }
            currentName = null;
            currentType = null;
            currentLimitingExpression = null;
        }

        @Override
        public void exitVarlist(@NotNull VariableParserParser.VarlistContext ctx) {
            for(Map.Entry<String, limitingExpressionData> limitingExpression : m_variableLimitingExpressions.entrySet()) {
                String subtypeVar = limitingExpression.getValue().variableName;
                String subtypeExpression = limitingExpression.getValue().expression;
                if (m_variables.containsKey(limitingExpression.getValue().variableName)) {
                    throw new IllegalStateException("Reused variable name for subtype declaration!");
                }
                // Get the variable we are working on, and use it for the variable used in the subtype expression.  This
                // effectively means the expression can just be emitted into the verification system to limit it.
                m_variables.put(subtypeVar, m_variables.get(limitingExpression.getKey()));

                VariableCollection variableCollection = new VariableCollection(m_variables, m_usedEnumerationTypes);
                m_variables.get(limitingExpression.getKey()).type().setSubtypePredicate(PVSSimpleParser.parsePVSCode(variableCollection, subtypeExpression));

                // Undo the variables hack, to avoid having unused variables bleed into the verification scripts!
                m_variables.remove(subtypeVar);
            }

            m_variableCollection = new VariableCollection(m_variables, m_usedEnumerationTypes);
        }

        private Map<String, Variable>  m_variables = new LinkedHashMap<String, Variable>();
        private Set<EnumerationVariableType> m_usedEnumerationTypes = new HashSet<EnumerationVariableType>();
        public VariableCollection m_variableCollection;

        private Map<String, limitingExpressionData> m_variableLimitingExpressions = new HashMap<String, limitingExpressionData>();
    }
    private static class limitingExpressionData {
        public String expression;
        public String variableName;
    }
}
